@@ -5,7 +5,9 @@
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2010-04-10 (displayInfo: get version from plist and date from __DATE__)
+ * modified: 2010-07-04 (svg stuff added)
+ *           2010-06-30 (-changeSafeType: TIFF added)
+ *           2010-04-10 (displayInfo: get version from plist and date from __DATE__)
  *           2010-01-12 (take snapshot of open documents and their positions)
  *           2009-06-24
  *           2009-03-27 (-sendEvent: ',' -> '.' on Apple)
@@ -47,6 +49,7 @@
 #include "DocView.h"
 #include "DocWindow.h"
 #include "PreferencesMacros.h"
+#include "UpdateController.h"
 
 #include "DINImportSub.h"
 #include "DXFImportSub.h"
@@ -54,6 +57,7 @@
 #include "HPGLImportSub.h"
 #include "PSImportSub.h"
 #include "Type1ImportSub.h"
+//#include "SVGImportSub.h"
 
 @implementation App
 
@@ -207,13 +211,13 @@
  * modified: 2005-09-05 (load module only once)
  */
 - (void)loadModules
-{   NSBundle		*mainBundle = [NSBundle mainBundle], *bundle;
-    Class		bundleClass;
-    NSString		*path;
-    int			i, j;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    BOOL		camModuleLoaded = NO;
-    NSMutableArray	*loadedFiles = [NSMutableArray array];
+{   NSBundle        *mainBundle = [NSBundle mainBundle], *bundle;
+    Class           bundleClass;
+    NSString        *path;
+    int             i, j;
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
+    BOOL            camModuleLoaded = NO;
+    NSMutableArray  *loadedFiles = [NSMutableArray array];
 
     /* load modules from application bundle
      * FIXME: The CAM module should be integrated below (case 0)
@@ -224,9 +228,9 @@
     path = [mainBundle pathForResource:@"CAM" ofType:@"bundle"];
     if ( (bundle = [NSBundle bundleWithPath:path]) )
     {
-        bundleClass = [bundle principalClass];	// controller
-        [modules addObject:bundle];		// our loaded modules
-        [bundleClass instance];			// create instance
+        bundleClass = [bundle principalClass];  // controller
+        [modules addObject:bundle];             // our loaded modules
+        [bundleClass instance];                 // create instance
         camModuleLoaded = YES;
     }
 
@@ -253,12 +257,12 @@
             if (camModuleLoaded && [file hasPrefix:@"Cut."])	// either CAM or Cut, not both
                 continue;
             if ( [file hasSuffix:@".bundle"] &&
-                 ![loadedFiles containsObject:file] &&	// already loaded ?
+                 ![loadedFiles containsObject:file] &&  // already loaded ?
                  (bundle = [NSBundle bundleWithPath:[path stringByAppendingPathComponent:file]]) )
             {
-                bundleClass = [bundle principalClass];	// controller
-                [modules addObject:bundle];		// our loaded modules
-                [bundleClass instance];			// create instance
+                bundleClass = [bundle principalClass];  // controller
+                [modules addObject:bundle];             // our loaded modules
+                [bundleClass instance];                 // create instance
                 [loadedFiles addObject:file];
             }
         }
@@ -340,12 +344,13 @@
     document = [Document new];
 }
 
-/* modified: 2009-02-25 (fileNameLC)
+/* modified: 2010-09-10 (defaultManager renamed to fileManager)
+ *           2009-02-25 (fileNameLC)
  */
 - (id)listFromFile:(NSString*)fileName
 {   NSArray         *list = nil;
     NSString        *path, *name;
-    NSFileManager   *defaultManager = [NSFileManager defaultManager];
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
     NSString        *fileNameLC = [fileName lowercaseString];
 
     //if ( [fileName rangeOfString:@".dxf" options:NSCaseInsensitiveSearch].length )
@@ -366,11 +371,13 @@
         /* load parameter file
          * 1st try it in the users home library then try it in /LocalLibrary
          */
-        name = [Prefs_HPGLParmsFileName stringByAppendingPathExtension:DEV_EXT];
+        name = Prefs_HPGLParmsFileName;
+        if ( ![name length] ) name = @"hpgl_8Pen";  // workaround GNUstep issue with registering defaults
+        name = [name stringByAppendingPathExtension:DEV_EXT];
         path = vhfPathWithPathComponents(userLibrary(), HPGLPATH, name, nil);
-        if ( ![defaultManager fileExistsAtPath:path] )
+        if ( ![fileManager fileExistsAtPath:path] )
         {   path = vhfPathWithPathComponents(localLibrary(), HPGLPATH, name, nil);
-            if ( ![defaultManager fileExistsAtPath:path] )
+            if ( ![fileManager fileExistsAtPath:path] )
             {	NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
             }
@@ -386,17 +393,19 @@
         [hpglImport release];
     }
     else if ( [fileNameLC hasSuffix:@".ger"] || [fileNameLC hasSuffix:@".gerber"] )
-    {   id	gerberImport;
-        NSData	*data;
+    {   id      gerberImport;
+        NSData  *data;
 
         /* load parameter file
          * 1st try it in the users home library then try it in /LocalLibrary
          */
-        name = [Prefs_GerberParmsFileName stringByAppendingPathExtension:DEV_EXT];
+        name = Prefs_GerberParmsFileName;
+        if ( ![name length] ) name = @"gerber"; // workaround GNUstep issue with registering defaults
+        name = [name stringByAppendingPathExtension:DEV_EXT];
         path = vhfPathWithPathComponents(userLibrary(), GERBERPATH, name, nil);
-        if ( ![defaultManager fileExistsAtPath:path] )
+        if ( ![fileManager fileExistsAtPath:path] )
         {   path = vhfPathWithPathComponents(localLibrary(), GERBERPATH, name, nil);
-            if ( ![defaultManager fileExistsAtPath:path] )
+            if ( ![fileManager fileExistsAtPath:path] )
             {	NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
             }
@@ -441,9 +450,9 @@
         [gerberImport release];
     }
     else if ( [fileNameLC hasSuffix:@".drl"] || [fileNameLC hasSuffix:@".din"] )
-    {   id		dinImport;
-        NSData		*data;
-        NSString	*devFileName;
+    {   id          dinImport;
+        NSData      *data;
+        NSString    *devFileName;
 
         /* look for parameter file
          * 1st try it in the users home library then try it in /LocalLibrary
@@ -452,9 +461,9 @@
         {
             name = [devFileName stringByAppendingPathExtension:DEV_EXT];
             path = vhfPathWithPathComponents(userLibrary(), DINPATH, name, nil);
-            if ( ![defaultManager fileExistsAtPath:path] )
+            if ( ![fileManager fileExistsAtPath:path] )
             {   path = vhfPathWithPathComponents(localLibrary(), DINPATH, name, nil);
-                if ( ![defaultManager fileExistsAtPath:path] )
+                if ( ![fileManager fileExistsAtPath:path] )
                 {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                     return nil;
                 }
@@ -474,6 +483,13 @@
         list = [[DINImportSub layerListFromGraphicList:list] retain];
         [dinImport release];
     }
+    /*else if ( [fileNameLC hasSuffix:@".svg"] )
+    {	SVGImport   *svgImport = [[SVGImportSub allocWithZone:[self zone]] init];
+        NSData      *data = [NSData dataWithContentsOfFile:fileName];
+
+        list = [[svgImport importSVG:data] retain]; // get list of graphic-objects from import
+        [svgImport release];
+    }*/
     else if ( [fileNameLC hasSuffix:@".font"] || [fileNameLC hasSuffix:@".pfa"] || [fileNameLC hasSuffix:@".pfb"] )
     {   id          fontObject, type1Import;
         NSData		*data;
@@ -597,7 +613,7 @@
             [self setCurrentDocument:nil];
             [document setName:UNTITLED_STRING andDirectory:[self currentDirectory]];
             [document setDirty:YES];
-            [document setFontObject:list]; // font daten + font list hinzufˆgen
+            [document setFontObject:list]; // add font data, font list
         }
         /* imports with complete layerList or a single list of objects */
         else
@@ -622,7 +638,7 @@ typedef enum
 {   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                               @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
-                                              @"font", @"pfa", @"din", @"drl", nil];
+                                              @"font", @"pfa", @"din", @"drl", @"svg", nil];
     NSString    *fileName;
     id          openPanel = [NSOpenPanel openPanel];
     static      NSString	*openDir = @"";
@@ -709,7 +725,7 @@ typedef enum
 {   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                   @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                   @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
-                                  @"font", @"pfa", @"din", @"drl", nil];
+                                  @"font", @"pfa", @"din", @"drl", @"svg", nil];
     NSArray         *fileNames;
     id              openPanel = [NSOpenPanel openPanel];
     int             i, cnt;
@@ -753,8 +769,9 @@ typedef enum
         case 2: [savePanel setRequiredFileType:GERBER_EXT]; break;
         case 3: [savePanel setRequiredFileType:DXF_EXT]; break;
         case 4: [savePanel setRequiredFileType:HPGL_EXT]; break;
-        case 5: [savePanel setRequiredFileType:FONT_EXT]; break;
-        case 6: [savePanel setRequiredFileType:DIN_EXT]; break;
+        case 5: [savePanel setRequiredFileType:TIFF_EXT]; break;
+        case 6: [savePanel setRequiredFileType:FONT_EXT]; break;
+        case 7: [savePanel setRequiredFileType:DIN_EXT]; break;
         default: NSLog(@"App, changeFileType: Unknown file type");
     }
 }
@@ -857,7 +874,7 @@ typedef enum
 
 /* Snapshots
  * created:  2010-01-12
- * modified: 2010-04-20
+ * modified: 2010-04-23
  */
 - (void)takeSnapshot:sender
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
@@ -881,13 +898,47 @@ typedef enum
                 [snapArray addObject:[NSArray arrayWithObjects:[doc filename], rString, nil]];
         }
     }
-    [defaults setObject:snapArray forKey:@"snapShotDocuments"];
+
+    {   NSString    *path = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".snapshots", nil);
+        NSString    *file, *name = @"Default.plist";
+        NSSavePanel *savePanel = [NSSavePanel savePanel];
+
+        if ( ! [[NSFileManager defaultManager] fileExistsAtPath:path])
+            [[NSFileManager defaultManager] createDirectoryAtPath:path attributes:nil];
+        file = vhfPathWithPathComponents(path, name, nil);
+
+        [savePanel setRequiredFileType:@"plist"];
+        //[savePanel setCanCreateDirectories:NO];
+        if ( [savePanel runModalForDirectory:path file:name] )
+            file = [savePanel filename];
+        else
+            return;
+        if ( [file rangeOfString:name].length )
+            [defaults setObject:snapArray forKey:@"snapShotDocuments"];
+
+        [snapArray writeToFile:file atomically:YES];
+    }
 }
 - (void)restoreSnapshot:sender
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
     NSArray         *snapArray = [defaults objectForKey:@"snapShotDocuments"];
     int             i;
 
+    if ( [sender isKindOfClass:[NSMenuItem class]] )
+    {   NSString    *path = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".snapshots", nil);
+        NSString    *file, *name = @"Default.plist";
+        NSOpenPanel *openPanel = [NSOpenPanel openPanel];
+
+        file = vhfPathWithPathComponents(path, name, nil);
+        [openPanel setRequiredFileType:@"plist"];
+        if ( [openPanel runModalForDirectory:path file:name] )
+            file = [openPanel filename];
+        else
+            return;
+        if ( ! (snapArray = [NSArray arrayWithContentsOfFile:file]) )
+            snapArray = [defaults objectForKey:@"snapShotDocuments"];
+    }
+
     for (i=0; i<[snapArray count]; i++)
     {   NSArray *array = [snapArray objectAtIndex:i];
 
@@ -942,31 +993,10 @@ typedef enum
         NSString        *version = [infoDict objectForKey:@"CFBundleVersion"];
 
         if ( version )
-        {   char    *compileDate = __DATE__;    // Apr 10 2010
-            char    date[15];
+        {   NSString    *compileDate = [self compileDate];
 
-            if ( strlen(compileDate) == 11 )    // Apr 10 2010 -> 2010-04-10
-            {   NSArray *mArray;
-                char    mStr[4];
-                int     m;
-
-                strncpy(mStr, compileDate, 3); mStr[3] = 0;
-                mArray = [NSArray arrayWithObjects:@"Jan", @"Feb", @"Mar", @"Apr", @"May", @"Jun", @"Jul", @"Aug", @"Sep", @"Oct", @"Nov", @"Dec", nil];
-                if ( (m = [mArray indexOfObject:[NSString stringWithUTF8String:mStr]]) != NSNotFound )
-                {   m ++;
-                    strncpy(date, compileDate+7, 4);    // YYYY
-                    date[4] = '-';
-                    date[5] = (m >= 10) ? '1' : '0';
-                    if (m >= 10) m -= 10;
-                    date[6] = '0' + m;                  // MM
-                    date[7] = '-';
-                    strncpy(date+8, compileDate+4, 2);    // DD
-                    date[10] = 0;
-                    compileDate = date;
-                }
-            }
             if ( compileDate )
-                version = [version stringByAppendingFormat:@" (%s)", compileDate];
+                version = [version stringByAppendingFormat:@" (%@)", compileDate];
             [infoVersionNo setStringValue:version]; // ex: 3.9.1 pre 1 (2010-02-13)
         }
     }
@@ -983,6 +1013,47 @@ typedef enum
     [self displayInfo];
     [infoPanel makeKeyAndOrderFront:sender];
 }
+- (NSString*)version
+{   NSDictionary    *infoDict = [[NSBundle mainBundle] infoDictionary];
+    NSString        *version;
+
+    version = [infoDict objectForKey:@"CFBundleVersion"];   // Apple
+    if ( ! version )
+        version = [infoDict objectForKey:@"NSVersion"];     // GNUstep
+    if ( ! version )
+        version = infoVersionNo;
+    return version;
+}
+- (NSString*)compileDate
+{   char    *compileDate = __DATE__;    // Apr 10 2010
+    char    date[15];
+
+    if ( strlen(compileDate) == 11 )    // Apr  2 2010 -> 2010-04-02
+    {   NSArray *mArray;
+        char    mStr[4];
+        int     m;
+
+        strncpy(mStr, compileDate, 3); mStr[3] = 0;
+        mArray = [NSArray arrayWithObjects:@"Jan", @"Feb", @"Mar", @"Apr", @"May", @"Jun", @"Jul", @"Aug", @"Sep", @"Oct", @"Nov", @"Dec", nil];
+        if ( (m = [mArray indexOfObject:[NSString stringWithUTF8String:mStr]]) != NSNotFound )
+        {   m ++;
+            strncpy(date, compileDate+7, 4);                            // YYYY
+            date[4] = '-';
+            date[5] = (m >= 10) ? '1' : '0';                            // MM
+            if (m >= 10) m -= 10;
+            date[6] = '0' + m;
+            date[7] = '-';
+            date[8] = (compileDate[4] == ' ') ? '0' : compileDate[4];   // DD
+            date[9] = compileDate[5];
+            date[10] = 0;
+            compileDate = date;
+        }
+    }
+    if ( compileDate )
+        return [NSString stringWithCString:compileDate encoding:NSASCIIStringEncoding];
+    else
+        return nil;
+}
 - (id)infoVersionNo     { return infoVersionNo; }   // ex: 3.9.0
 - (id)infoVersionText	{ return kindOfVersion; }   // ex: "Licensed version"
 - (id)infoSerialText	{ return serialNumber; }    // ex: 020001
@@ -1004,6 +1075,41 @@ typedef enum
     return preferencesPanel;
 }
 
+- (void)checkForUpdate:sender
+{
+#ifdef __APPLE__
+    [[UpdateController sharedInstance] checkForUpdates:sender];
+#else   // GNUstep: TODO
+    {   NSString    *site;
+
+        site = NSLocalizedString(@"http://www.cenon.info/news/news_gb.html", News);
+        //site = NSLocalizedString(@"http://www.cenon.info/dApple_gb.html", Download);
+        [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:site]];
+    }
+#endif
+}
+
+/* Show Web Page
+ * created: 2010-05-19
+ */
+- (void)showWebPage:(id)sender
+{   NSString    *site;
+    int         tag = [sender tag];
+
+	if ([sender isKindOfClass:[NSMatrix class]])
+		tag = [sender selectedTag];
+	switch (tag)
+    {
+        default: site = NSLocalizedString(@"http://www.cenon.info", Web Site); break;
+        case 1:  site = NSLocalizedString(@"http://www.cenon.info/support_faq_gb.html", Web FAQ); break;
+        case 2:  site = NSLocalizedString(@"http://www.cenon.info/releaseNotes_gb.html", Web Releae Notes); break;
+        case 3:  site = NSLocalizedString(@"mailto:service@vhf.de?subject=Cenon%20Feedback", eMail Address); break;
+        case 4:  site = NSLocalizedString(@"http://www.cenon.info/news/news_gb.html", News); break;
+                 //site = NSLocalizedString(@"http://www.cenon.info/dApple_gb.html", Download); break;
+    }
+	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:site]];
+}
+
 /* load PDF documentation
  * Note: each module can add a help menu entry to load it's docu
  * modified: 2007-07-22
@@ -1366,6 +1472,7 @@ typedef enum
                     if (!inspectorPanel)
                         [self showInspectorPanel:self];
                     [inspectorPanel setLevelAt:i-1];
+                    return;
                 }
             }
         }
