@@ -1,11 +1,12 @@
 /* Document.m
  * Cenon document class
  *
- * Copyright (C) 1996-2008 by vhf interservice GmbH
+ * Copyright (C) 1996-2010 by vhf interservice GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2008-10-21 (clean-up)
+ * Modified: 2010-06-30 (-saveTiff added)
+ *           2008-10-21 (clean-up)
  *           2008-07-24 (document units, update units)
  *
  * This program is free software; you can redistribute it and/or
@@ -49,14 +50,14 @@
 #    include "GSPropertyListSerialization.h"
 #endif
 
-#define XOFFSET		5.0 	// Offset of subsequent windows
-#define YOFFSET 	-20.0
-#define	MAXWINWIDTH	900	// maximum size of the window
-#define	MAXWINHEIGHT	750
+#define XOFFSET             5.0     // Offset of subsequent windows
+#define YOFFSET             -20.0
+#define	MAXWINWIDTH         900     // maximum size of the window
+#define	MAXWINHEIGHT        750
 
-#define DOCUMENT_VERSION	102
-#define DOCUMENT_NAME		@"document"
-#define OUTPUT_NAME		@"output"
+#define DOCUMENT_VERSION    102
+#define DOCUMENT_NAME       @"document"
+#define OUTPUT_NAME         @"output"
 
 /* Private methods */
 @interface Document(PrivateMethods)
@@ -64,6 +65,7 @@
 - (void)initializePrintInfo;
 - (BOOL)setWindowSize:(NSSize)size;
 - (BOOL)loadOutputListsFromFile:(const char*)fileName;
+- (BOOL)saveTiff:(NSString*)filename;
 - (BOOL)saveEPS:(NSString*)filename;
 - (BOOL)saveGerber:(NSString*)filename;
 - (BOOL)saveHPGL:(NSString*)filename;
@@ -1225,6 +1227,8 @@ static NSMutableArray *zoneList = nil;
             [self saveDXF:path];
         else if ([[path pathExtension] isEqual:FONT_EXT])
             [self saveFont:path];
+        else if ([[path pathExtension] isEqual:TIFF_EXT])
+            [self saveTiff:path];
     }
 
     return NO;
@@ -1299,6 +1303,167 @@ static NSMutableArray *zoneList = nil;
     return savedOk;
 }
 
+/* created: 2010-06-29
+ */
+- (BOOL)saveTiff:(NSString*)filename
+{   NSArray             *layerList = [[self documentView] layerList];
+    NSRect              conRect, rect, bbox = NSZeroRect;
+    NSBitmapImageRep	*bitmapImage;
+    NSImage             *img = nil;
+    NSData              *tdata = nil;
+#ifdef __APPLE__
+    NSGraphicsContext   *gContext;
+    CGContextRef        cgContext=nil;
+#else
+    id                  win = nil, conView = nil;
+#endif
+
+    /* only all visible Layers */
+    {   int	l, cnt = [layerList count];
+
+        for (l=0; l<cnt; l++)
+        {	LayerObject	*layerObject = [layerList objectAtIndex:l];
+
+            if ( [layerObject state] ) // visible
+                bbox = NSUnionRect([[self documentView] boundsOfArray:[layerObject list]], bbox);
+        }
+    }
+
+    conRect.origin.x = conRect.origin.y = 10.0;
+    conRect.size.width = bbox.size.width;
+    conRect.size.height = bbox.size.height;
+
+    rect.origin = NSZeroPoint;
+    rect.size.width = (int)(conRect.size.width);
+    rect.size.height = (int)(conRect.size.height);
+
+#ifdef __APPLE__    // Apple
+    [NSGraphicsContext saveGraphicsState];
+    bitmapImage = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:NULL
+                                                          pixelsWide:rect.size.width
+                                                          pixelsHigh:rect.size.height
+                                                       bitsPerSample:8
+                                                     samplesPerPixel:4
+                                                            hasAlpha:YES isPlanar:NO // YES - geht nicht
+                                                      colorSpaceName:NSCalibratedRGBColorSpace
+                                                        bitmapFormat:0
+                                                         bytesPerRow:rect.size.width*4
+                                                        bitsPerPixel:32];
+
+    /* this is the important code for the correct Color Values - need kCGColorSpaceGenericRGB since 10.4 */
+    {   CGColorSpaceRef colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
+
+        cgContext = CGBitmapContextCreate([bitmapImage bitmapData], rect.size.width, rect.size.height, 8, rect.size.width*4,
+                                          colorSpace, kCGImageAlphaPremultipliedLast);
+        gContext = [NSGraphicsContext graphicsContextWithGraphicsPort:cgContext
+                                                              flipped:[[NSGraphicsContext currentContext] isFlipped]];
+        CGColorSpaceRelease(colorSpace);
+    }
+    [NSGraphicsContext setCurrentContext:gContext];
+#else               // OpenStep and GNUstep
+    /* build an offscreen window
+     * in which we draw the scaled black copy of original path on white background NSBackingStoreBuffered NSBackingStoreRetained
+     */
+    win = [[NSWindow alloc] initWithContentRect:conRect styleMask:NSBorderlessWindowMask backing:NSBackingStoreRetained defer:NO];
+    [win setDepthLimit:NSBestDepth(NSCalibratedWhiteColorSpace, 8, 8, YES, NULL)];
+
+    [win setBackgroundColor:[NSColor whiteColor]];
+#if DEBUG_TIFFEXPORT
+    [win makeKeyAndOrderFront:self]; /* weglassen wenn nicht gesehen werden soll */
+#endif
+    conView = [win contentView];
+    [conView display];
+
+    /* now draw our black scaled path in view
+     */
+    [conView lockFocus];
+#endif
+
+#ifdef __APPLE__
+    /* draw color shading without antialiasing */
+    [[NSGraphicsContext currentContext] setShouldAntialias:Prefs_Antialias];
+
+    /* first draw white rectangle in full size */
+    {   VRectangle  *r = [VRectangle rectangle];
+
+        [r setFilled: YES];
+        [r setFillColor:[NSColor whiteColor]];
+        [r setVertices:rect.origin :NSMakePoint(rect.size.width, rect.size.height)];
+        [r drawWithPrincipal:nil];
+    }
+#endif
+
+    /* draw all visible Layers */
+    {   int     l, cnt = [layerList count];
+        NSPoint movePt = NSMakePoint(-bbox.origin.x, -bbox.origin.y);
+        NSPoint moveBackPt = bbox.origin; // NSMakePoint(-bbox.origin.x, -bbox.origin.y);
+
+        for (l=0; l<cnt; l++)
+        {	LayerObject	*layerObject = [layerList objectAtIndex:l];
+
+            if ( [layerObject state] ) // visible                
+            {   NSMutableArray	*llist = [layerObject list];
+                int i, lcnt = [llist count];
+
+                for (i=0; i<lcnt; i++)
+                {   VGraphic    *gr = [llist objectAtIndex:i];
+
+                    [gr moveBy:movePt];
+                    [gr drawWithPrincipal:nil];
+                    [gr moveBy:moveBackPt];
+                }
+            }
+        }
+    }
+
+    /* get pixel information into data (char string) */
+
+#if !defined(__APPLE__)    // OpenStep 4.2 and GNUstep
+    bitmapImage = [[NSBitmapImageRep allocWithZone:[self zone]] initWithFocusedViewRect:rect];
+#endif
+
+    /* generate Image from bitmapRep */
+    img = [[NSImage alloc] initWithSize:rect.size];
+    [img addRepresentation:bitmapImage];
+
+    /* save image */
+    if ( (tdata = [img TIFFRepresentation]) )
+    {   [tdata writeToFile:filename atomically:YES];
+        return YES;
+    }
+
+#ifdef __APPLE__
+    [NSGraphicsContext restoreGraphicsState];
+    CGContextRelease(cgContext);
+#else
+    [conView unlockFocus];
+    [win close];
+#endif
+
+    [img release];
+    [bitmapImage release];
+    return NO;
+}
+#if 0
+/*{   NSPasteboard	*pb = [NSPasteboard pasteboardWithName:@"pasteboard"];
+    NSImage         *img;
+    NSData          *tdata = nil;
+    NSRect          bbox;
+
+    bbox = [[self documentView] boundsOfArray:[[self documentView] layerList]];
+
+    [[self documentView] writeEPSInsideRect:bbox toPasteboard:pb]; // did not work (antialiased and no white)
+    img = [[NSImage alloc] initWithPasteboard:pb];
+    if ( (tdata = [img TIFFRepresentation]) )
+    {   [tdata writeToFile:filename atomically:YES];
+        return YES;
+    }
+    [img release];
+    [pb releaseGlobally];
+    return NO;
+}*/
+#endif
+
 - (void)printSeparation:(NSPrintOperation*)op
 {   NSColor             *sepColor;
     NSPrintOperation    *sop=nil;
@@ -1854,18 +2019,18 @@ static NSMutableArray *zoneList = nil;
     if ( dirty )
         switch (NSRunAlertPanel(CLOSEWINDOW_STRING, SAVECHANGES_STRING, SAVE_STRING, DONTSAVE_STRING, CANCEL_STRING, [sender title]))
         {
-            case NSAlertDefaultReturn:		// save
+            case NSAlertDefaultReturn:      // save
                 [self save];
                 break;
-            case NSAlertAlternateReturn:	// dont save
+            case NSAlertAlternateReturn:    // dont save
                 break;
-            default:				// cancel
+            default:                        // cancel
                 return NO;
         }
 
     for (l=[[[self documentView] layerList] count]-1; l>=0; l--)
         [[[[self documentView] layerList] objectAtIndex:l] setDirty:NO];
-
+    [window endEditingFor:nil]; // end editing of text
     [window setDocument:nil];
     [self autorelease];
 
@@ -1879,6 +2044,7 @@ static NSMutableArray *zoneList = nil;
 {   [window windowDidBecomeMain:notification]; }
 - (void)windowDidResignMain:(NSNotification *)notification
 {
+    [window endEditingFor:nil]; // end editing of text
     [[NSNotificationCenter defaultCenter] postNotificationName:DocWindowDidChange
                                                         object:nil];
 }
