@@ -6,7 +6,8 @@
  *           Ilonka Fleischmann
  *
  * created:  1996-05-03
- * modified: 2005-05-12
+ * modified: 2009-07-10 (// state.g = 0; -getLayerPolarity: ![scanner isAtEnd])
+ *           2006-03-24
  *
  * This file is part of the vhf Import Library.
  *
@@ -605,7 +606,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
                         default:
                             [self setLine:cList];
                     }
-                    state.g = 0;
+//                    state.g = 0;
                     break;
                 case 3: /* flash */
                     [self setPad:list];
@@ -892,7 +893,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
         else // the values follows directly behind , (comma) -> we must combine them with $1 values in macroData
         {   NSScanner	*mScanner = [NSScanner scannerWithString:macroData];
             NSString	*searchStr = [NSString stringWithFormat:@"%%AM%@", macroName];
-            NSRange	range;
+            NSRange	range, rangeHack;
             int		i, cnt = 0;
             float	vals[200]; // I hope no one will write an greater macro
 
@@ -914,10 +915,48 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
             }
             // replace $1, $2 $n with values from scanner
             range = [*macroStr rangeOfString:@"="];
+            /* Eagle WorkAround */
+            rangeHack = [*macroStr rangeOfString:@"X$"];
             for (i=0; i<cnt; i++)
             {   NSString	*from = [NSString stringWithFormat:@"$%d", i+1];
                 NSString	*to = [NSString stringWithFormat:@"%.3f", vals[i]];
 
+                /* Eagle WorkAround */
+                if ( rangeHack.length )
+                {   NSScanner	*mcrScanner = [NSScanner scannerWithString:*macroStr];
+                    NSString	*string;
+
+                    NSLog(@"Eagle WorkAround wurde aktiviert\n");
+                    /*  * 5,1,8,0,0,1.08239X$1,22.5  - werte zwischen den Kommas */
+                    /* we must remove the float before/and the X */
+                    /* and we must set the last float to 0 - rotation of polygon */
+
+                    while ( ![mcrScanner isAtEnd] )
+                    {
+                        [mcrScanner scanUpToString:@"," intoString:&string];
+                        rangeHack = [string rangeOfString:@"X$"];
+                        /* this is our string part - wert zwischen den kommas */
+                        if ( rangeHack.length )
+                        {   NSScanner		*mmScanner = [NSScanner scannerWithString:string];
+                            NSMutableString	*fromHack = [NSMutableString stringWithString:string], *toHack;
+                            NSString		*restString;
+
+                            [mcrScanner scanUpToString:@"*" intoString:&restString]; // ,22.5 rotation
+                            [fromHack appendString:restString];
+
+                            [mmScanner scanUpToString:@"$" intoString:NULL]; // bis $
+                            [mmScanner scanUpToString:@"," intoString:&toHack]; // bis , -> $1 oder $3
+
+                            /* and we must set the last float to 0 - rotation of polygon */
+                            toHack = [NSMutableString stringWithString:toHack];
+                            [toHack appendString:@",0"]; // no rotation
+                            *macroStr = [*macroStr stringByReplacing:fromHack by:toHack];
+                            break;
+                        }
+                        [mcrScanner scanString:@"," intoString:NULL];
+                    }
+                }
+
                 if ( !range.length )
                     *macroStr = [*macroStr stringByReplacing:from by:to];
                 else
@@ -965,7 +1004,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
 - (BOOL)getLayerPolarity:(NSScanner*)scanner
 {
     // until %
-    while ( ![scanner scanString:ops.RS274X intoString:NULL] )
+    while ( ![scanner scanString:ops.RS274X intoString:NULL] && ![scanner isAtEnd] )
     {
         if ( [scanner scanString:@"LP" intoString:NULL] ) // no layer polarity
         {
@@ -1559,7 +1598,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
         [self addFillPath:dList toList:mList];
         dList=[[[self allocateList] init] autorelease];
 
-        // set 4 points of horicontal (   ) line
+        // set 4 points of horicontal line
         p[0].x = cx - lenCross/2.0; p[0].y = cy - wCross/2.0;
         p[1].x = p[0].x + lenCross; p[1].y = p[0].y;
         p[2].x = p[1].x; p[2].y = p[1].y + wCross;
@@ -1656,7 +1695,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
 
         [self addFillPath:dList toList:mList];
 
-        // set 4 points of horicontal (   ) line
+        // set 4 points of horicontal ( –  ) line
         p[0].x = cx - (dOut2 + 1.0); p[0].y = cy - (wCross/2.0 + 1.0);
         p[1].x = p[0].x + dOut + 2.0; p[1].y = p[0].y;
         p[2].x = p[1].x; p[2].y = p[1].y + wCross + 2.0;
@@ -1916,7 +1955,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
 
 /*
  * created:   07.05.93
- * modified:  17.06.93 05.05.96
+ * modified:  17.06.93 05.05.96 2009-07-10 (//state.g = 0)
  * purpose:   set arc ccw
  * parameter: g
  */
@@ -1925,7 +1964,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
     float		ba, ea, angle;
     NSDictionary	*tool = [tools objectAtIndex:state.tool];
 
-    state.g = 0; // else we go on with an arc
+//    state.g = 0; // else we go on with an arc
 
     if ( !state.path )
         state.width = [[tool objectForKey:@"width"] floatValue];
@@ -1950,7 +1989,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
         angle = ea - ba;
 
     if ( Diff(Abs(angle), 0.0) <= TOLERANCE || Diff(Abs(angle), 360.0) <= TOLERANCE )
-        angle = (angle > 0) ? (angle = 360.0) : (angle = -360.0);
+        angle = (angle > 0) ? 360.0 : -360.0;
 
     if ( state.ipolFull && Diff(angle, 360.0) <= TOLERANCE )
     {
